<?php
/**
 * Copyright (c) 2020, Nosto Solutions Ltd
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * @author Nosto Solutions Ltd <contact@nosto.com>
 * @copyright 2020 Nosto Solutions Ltd
 * @license http://opensource.org/licenses/BSD-3-Clause BSD 3-Clause
 *
 */

namespace Nosto\Tagging\Model\Product\Cache;

use Magento\Framework\Stdlib\DateTime\TimezoneInterface;
use Magento\Store\Api\Data\StoreInterface;
use Nosto\Tagging\Model\Product\Builder as NostoProductBuilder;
use Nosto\Tagging\Model\Product\BuilderTrait;
use Nosto\Tagging\Model\Product\Cache as CacheModel;
use Nosto\Tagging\Model\Product\CacheFactory;
use Nosto\Types\Product\ProductInterface as NostoProductInterface;

class CacheBuilder
{
    use BuilderTrait {
        BuilderTrait::__construct as builderTraitConstruct; // @codingStandardsIgnoreLine
    }

    /** @var CacheFactory  */
    private $cacheFactory;

    /** @var NostoProductBuilder */
    private $nostoProductBuilder;

    /** @var TimezoneInterface */
    private $magentoTimeZone;

    /**
     * Builder constructor.
     * @param CacheFactory $NostoCacheFactory
     * @param TimezoneInterface $magentoTimeZone
     */
    public function __construct(
        CacheFactory $NostoCacheFactory,
        TimezoneInterface $magentoTimeZone
    ) {
        $this->cacheFactory = $NostoCacheFactory;
        $this->magentoTimeZone = $magentoTimeZone;
    }

    /**
     * @param NostoProductInterface $product
     * @param StoreInterface $store
     * @return CacheModel
     */
    public function build(
        NostoProductInterface $product,
        StoreInterface $store
    ) {
        $productIndex = $this->cacheFactory->create();
        $productIndex->setProductId($product->getProductId());
        $productIndex->setCreatedAt($this->magentoTimeZone->date());
        $productIndex->setIsDirty(false);
        $productIndex->setUpdatedAt($this->magentoTimeZone->date());
        $productIndex->setStore($store);
        return $productIndex;
    }
}
